﻿using System;
using System.Text;
using System.Net;
using System.IO;
using System.Data.OleDb;
using System.Xml;
using System.Xml.Xsl;
using System.Xml.Linq;
using System.Linq;
using System.Windows;
using System.Data;
using System.Web;
using System.Windows.Forms;
using System.Collections;
using System.Collections.Generic;
using System.Data.Odbc;

namespace SpiderRates
{
    class HotelInterfaceTools
    {
        public static string TimeStamp()
        {
            DateTime dtNow = DateTime.Now;
            return String.Format("{0:s}", dtNow);
        }

        public static void WriteLog(string msg)
        {
            try
            {
                using (StreamWriter w = File.AppendText(getPath(HotSpiderSettings.localPath, "LOG") + "spider.Log"))
                {
                    w.WriteLine(DateTime.Now + ", " + msg);
                }
            }
            catch (IOException ex)
            {
                MessageBox.Show("Ein Problem beim Schreiben in die Log-Datei ist aufgetreten. Bitte wenden sie sich an den Support.", "Hotel-Spider - Fehlermeldung!");
                CloseApplication();
            }
        }

        public static string RSSuccess(string xmlString)
        {
            Boolean error = true;
            string result = "";
            // Create an XmlReader
            if (xmlString != null)
            {
                using (XmlReader xmlReader = XmlReader.Create(new StringReader(xmlString)))
                {
                    try
                    {
                        // Parse the String and return each of the nodes.
                        while ((xmlReader.Read()) && (error))
                        {
                            if ((xmlReader.NodeType == XmlNodeType.Element) && xmlReader.Name.Equals("Success"))
                            {
                                result = xmlReader.Name;
                                error = false;
                            }
                            else
                            {
                                if ((xmlReader.NodeType == XmlNodeType.Element) && xmlReader.Name.Equals("Error"))
                                {
                                    result = xmlReader.Name;
                                }
                                if (xmlReader.NodeType == XmlNodeType.Text)
                                {
                                    result = xmlReader.Name + xmlReader.Value;
                                }
                            }
                        }
                    }
                    catch (XmlException ex)
                    {
                        MessageBox.Show("Aktueller Preiscode wurde nicht aktuallisiert. Bitte versuchen sie es später erneut. Wenn das Problem weiterhin auftritt, dann wenden sie sich bitte an den Support. RuntimeError: " + ex + "SpiderRates-Meldung: " + result, "Hotel-Spider - Fehlermeldung!");
                        CloseApplication();
                    }
                }
            } // End of If
            return result;
        }

        // Für vorzeitiges Beenden des Programms
        public static void CloseApplication()
        {
            if (System.Windows.Forms.Application.MessageLoop)
            {
                // WinForms app
                System.Windows.Forms.Application.Exit();
            }
            else
            {
                // Console app
                System.Environment.Exit(1);
            }
        }

        public static void prepaireXMLforString(string path)
        {
            var lines = File.ReadAllLines(path);
            File.WriteAllLines(path, lines.Skip(1).ToArray());
        }

        public static string MakeXmlValid(string xmlString)
        {
            return @"<?xml version=""1.0""?> <InternetData> " + xmlString + " </InternetData>";
        }

        public static string getPath(string configPath, string kindOfPath)
        {
            int counter = 0;
            string path = null;
            string[] strArray = null;
            string line = null;
            string configHoRes = configPath + "CONFIG.FPW";

            if (File.Exists(configHoRes))
            {
                System.IO.StreamReader file = new System.IO.StreamReader(configHoRes);
            
            while ((line = file.ReadLine()) != null)
            {
                strArray = line.Split('=');
                if (strArray[0] == "Default")
                {
                    if (kindOfPath.Equals("DBF") || kindOfPath.Equals("dbf") || kindOfPath.Equals("Dbf"))
                    {
                        path = strArray[1] + "\\dbf\\";
                    }
                    else if (kindOfPath.Equals("LOG") || kindOfPath.Equals("log") || kindOfPath.Equals("Log"))
                    {
                        path = strArray[1] + "\\ifc\\internet\\spider\\";
                    }
                }
                counter++;
            }
            file.Close();
            }
            else
            {
                MessageBox.Show("Die Datei: " + configHoRes + " wurde nicht gefunden. Wenden sie sich bitte an den Support.", "Hotel-Spider - Fehlermeldung!");
                CloseApplication();
            }
            return path;
        }

        public static string SendRQandGetRS(string soap, string uri)
        {
            // send request
            HttpWebRequest req = (HttpWebRequest)WebRequest.Create(uri);
            req.ServerCertificateValidationCallback += (sender, certificate, chain, sslPolicyErrors) => { return true; };
            req.ContentType = "application/soap+xml;";
            req.Method = "POST";

            using (Stream stm = req.GetRequestStream())
            {
                using (StreamWriter stmw = new StreamWriter(stm))
                {
                    try
                    {
                        stmw.Write(soap);
                    }
                    catch (WebException ex)
                    {
                        MessageBox.Show("Eine Verbindung mit dem Internet konnte nicht aufgebaut werden. Fehler: " + ex, "Hotel-Spider - Fehlermeldung!");
                        CloseApplication();
                    }
                }
            }

            // receive response and write to string
            WebResponse response = req.GetResponse();
            Stream responseStream = response.GetResponseStream();

            XmlTextReader reader = new XmlTextReader(responseStream);
            StringBuilder sb = new StringBuilder();
            try
            {
                while (reader.Read())
                {
                    sb.AppendLine(reader.ReadOuterXml());
                }
            }
            catch (XmlException ex)
            {
                MessageBox.Show("Beim Empfang der Preiscodes von Hotel-Spider ist ein Fehler aufgetreten. Bitte versuchen sie es später erneut. Wenn das Problem weiterhin auftritt, dann wenden sie sich bitte an den Support. RuntimeError: " + ex, "Hotel-Spider - Fehlermeldung!");
                CloseApplication();
            }
            return sb.ToString();
        }

        public static string ParseRatePlanXML(string xmlString)
        {
            StringBuilder output = new StringBuilder();
            bool isRatePlanDescription = false;
            bool isRateTierDescription = false;
            string ratePlanID = "";
            string currency = "EUR";
            string rateTierID = "";
            string rateTierDescription = "";
            string ratePlanDescription = "";
            string roomID = "";

            // Create an XmlReader
            //           XmlReaderSettings rs = new XmlReaderSettings();
            //           rs.CheckCharacters = false;
            using (XmlReader xmlReader = XmlReader.Create(new StringReader(xmlString))) //, rs))
            {
                XmlWriterSettings ws = new XmlWriterSettings();
                ws.Indent = true;
                ws.OmitXmlDeclaration = true;
                using (XmlWriter writer = XmlWriter.Create(output, ws))
                {
                    try
                    {
                        // Write the SpiderRooms root-element
                        writer.WriteStartElement("OnlineData");
                        // Parse the file.
                        while (xmlReader.Read())
                        {
                            if (xmlReader.NodeType == XmlNodeType.Element)
                            {
                                if (xmlReader.Name == "SellableProduct")
                                {
                                    // Write the table-element for the Grid-output
                                    writer.WriteStartElement("Table");
                                }

                                switch (xmlReader.Name)
                                {
                                    case "RatePlan":
                                        isRatePlanDescription = true;
                                        isRateTierDescription = false;
                                        break;                                  
                                    case "SellableProducts":
                                        isRateTierDescription = true;
                                        isRatePlanDescription = false;                                                                             
                                        break;                                   
                                }

                                if (xmlReader.HasAttributes)
                                {
                                    for (int attInd = 0; attInd < xmlReader.AttributeCount; attInd++)
                                    {
                                        xmlReader.MoveToAttribute(attInd);

                                        switch (xmlReader.Name)
                                        {

                                            case "RatePlanCode":
                                                ratePlanID = xmlReader.Value;
                                                break;                                           
                                            //case "CurrencyCode":
                                            //    currency = xmlReader.Value;
                                            //    break;
                                            case "InvCode":
//                                                roomID = xmlReader.Value;
                                                break;
                                            case "ID":
//                                                rateTierID = xmlReader.Value;
                                                break;
                                        } // End of switch
                                    }
                                    xmlReader.MoveToElement();
                                } // End of If element has attributes
                            } // End of If node is element

                            if (xmlReader.NodeType == XmlNodeType.Text)
                            {
                                if (isRatePlanDescription)
                                {
                                    ratePlanDescription = xmlReader.Value;
                                }
                                else if (isRateTierDescription)
                                {
//                                    rateTierDescription = xmlReader.Value;
                                }                                
                            } // End of IF Text       

                            if (xmlReader.NodeType == XmlNodeType.EndElement && xmlReader.Name.Equals("SellableProduct"))
                            {
                                writer.WriteStartElement("ra_ratcode");
                                writer.WriteString(ratePlanID);
                                writer.WriteEndElement();

                                writer.WriteStartElement("ra_priced");
                                writer.WriteString(ratePlanDescription);
                                writer.WriteEndElement();

                                writer.WriteStartElement("ra_curency");
                                writer.WriteString(currency);
                                writer.WriteEndElement();

                                writer.WriteStartElement("ra_ratier");
                                writer.WriteString(rateTierID);
                                writer.WriteEndElement();

                                writer.WriteStartElement("ra_ratierd");
                                writer.WriteString(rateTierDescription);
                                writer.WriteEndElement();

                                //writer.WriteStartElement("ra_roomID");
                                //writer.WriteString(roomID);
                                //writer.WriteEndElement();

                                writer.WriteEndElement();
                            }
                        }
                        // Write the close tag for the root element.
                        writer.WriteEndElement();
                    }
                    catch (XmlException ex)
                    {
                        MessageBox.Show("Beim Bearbeiten der Preiscodes von Hotel-Spider ist ein Fehler aufgetreten. Bitte versuchen sie es später erneut. Wenn das Problem weiterhin auftritt, dann wenden sie sich bitte an den Support. RuntimeError: " + ex, "Hotel-Spider - Fehlermeldung!");
                        CloseApplication();
                    }
                }
            }
            return output.ToString();
        }

        // Erkennt doppelte Elemente in XDocument
        private static IEnumerable<XNode> FilterDuplicates(IEnumerable<XNode> nodes)
        {
            foreach (XNode node in nodes.Where(n => !n.NodesBeforeSelf().Any(s => XNode.DeepEquals(n, s))))
            {
                switch (node.NodeType)
                {
                    case XmlNodeType.Element:
                        XElement elNode = node as XElement;
                        yield return new XElement(elNode.Name, elNode.Attributes(), FilterDuplicates(elNode.Nodes()));
                        break;
                    case XmlNodeType.Text:
                        yield return new XText(node as XText);
                        break;
                    case XmlNodeType.CDATA:
                        yield return new XCData(node as XCData);
                        break;
                    case XmlNodeType.Comment:
                        yield return new XComment(node as XComment);
                        break;
                    case XmlNodeType.ProcessingInstruction:
                        yield return new XProcessingInstruction(node as XProcessingInstruction);
                        break;
                }
            }
        }
        
        // Lösch doppelte Elemente
        public static void DeleteDoubleElements(string file)
        {
            XDocument input = XDocument.Load(file);
            XDocument output = new XDocument(
                FilterDuplicates(input.Nodes())
                );

            output.Save(file);
            prepaireXMLforString(file);
        }

        public static void DataSetIntoDBF(string path, string fileName, DataSet dataSet)
        {
            if (File.Exists(path + fileName + ".dbf"))
            {                                           
                string connectionString = "";

                //if (HotSpiderSettings.dBaseIV.Equals("T")) 
                //{
                //    connectionString = @"Provider=" + HotSpiderSettings.provider + ";Data Source=" + path + ";Extended Properties=" + HotSpiderSettings.extProperties; // dBase IV";    
                //} else if (HotSpiderSettings.vfp.Equals("T")) 
                //{
                    connectionString = "Provider=" + HotSpiderSettings.vfpProvider + ";Data Source=" + path + HotSpiderSettings.extended;
                //} else
                //{
                //    WriteLog("Wählen sie in der Datei Settings-Datei einen Datenbanktreiber aus (=T).");
                //    MessageBox.Show("Bitte wenden sie sich an den Support.", "Hotel-Spider - Fehlermeldung!");
                //    CloseApplication();
                //}
                
                using (OleDbConnection connection = new OleDbConnection(connectionString))
                using (OleDbCommand command = connection.CreateCommand())
                {
                    try
                    {
                        connection.Open();

                        foreach (DataRow row in dataSet.Tables[0].Rows)
                        {
                            string insertSql = "INSERT INTO " + fileName + " (ra_ratcode,ra_priced,ra_curency,ra_ratier,ra_ratierd) VALUES (";
                            
                            for (int i = 0; i < dataSet.Tables[0].Columns.Count; i++)
                            {
                                 insertSql = insertSql + "'" + ReplaceEscape(row[dataSet.Tables[0].Columns[i].ToString()].ToString()) + "',";
                            }
                            insertSql = insertSql.Substring(0, insertSql.Length - 1) + ")";
                            
                            command.CommandText = insertSql;
                            command.ExecuteNonQuery();
                        }
                    }
                    catch (OleDbException ex)
                    {
                        MessageBox.Show("Fehler beim Schreiben in die rates_S.dbf Datei: " + ex, "Hotel-Spider");
                    }
                    finally
                    {
                        connection.Close();
                    }
                }
            }
            else
            {
                MessageBox.Show("Die Datei " + fileName + " fehlt. Bitte wenden sie sich an den Support.", "Hotel-Spider - Fehlermeldung!");
                CloseApplication();
            }
        }

        private static string GetConnection(string path)
        {
            return "Provider=" + HotSpiderSettings.provider + ";Data Source=" + path + ";Extended Properties=" + HotSpiderSettings.extProperties;
        }

        public static string ReplaceEscape(string str)
        {
            str = str.Replace("'", "''");
            return str;
        }

        public static string ReplaceQuotes(string str)
        {
            str = str.Replace("\"", "");
            return str;
        }

        public static void DataTableToCsv(DataTable dataTable, string filePath)
        {
            StringBuilder fileContent = new StringBuilder();

            foreach (var col in dataTable.Columns)
            {
                fileContent.Append(col.ToString() + ",");
            }

            fileContent.Replace(",", System.Environment.NewLine, fileContent.Length - 1, 1);

            foreach (DataRow dr in dataTable.Rows)
            {

                foreach (var column in dr.ItemArray)
                {
                    fileContent.Append("\"" + column.ToString() + "\",");
                }

                fileContent.Replace(",", System.Environment.NewLine, fileContent.Length - 1, 1);
            }

            System.IO.File.WriteAllText(filePath, fileContent.ToString());
        }

        public static void CreateDBFTableOleDb(string path)
        {
            string connectionString = "";
                                                
            //if (HotSpiderSettings.dBaseIV.Equals("T")) 
            //{
            //    connectionString = @"Provider=" + HotSpiderSettings.provider + ";Data Source=" + path + ";Extended Properties=" + HotSpiderSettings.extProperties; // dBase IV";    
            //} else if (HotSpiderSettings.vfp.Equals("T")) 
            //{
                connectionString = "Provider="+ HotSpiderSettings.vfpProvider + "; Data Source=" + path + HotSpiderSettings.extended;
            //} else
            //{
            //    WriteLog("Wählen sie in der Datei spider.config einen Datenbanktreiber aus (=T).");
            //    MessageBox.Show("Bitte wenden sie sich an den Support.", "Hotel-Spider - Fehlermeldung!");
            //    CloseApplication();
            //}
              
            using (OleDbConnection connection = new OleDbConnection(connectionString))
            using (OleDbCommand command = connection.CreateCommand())
            {
                try
                {
                    connection.Open();
                    command.CommandText = "CREATE TABLE [rates_S] ([Ra_ratcode] CHAR(35) NOT NULL, [Ra_priced] CHAR(30) NOT NULL, [Ra_curency] CHAR(4) NOT NULL, [Ra_ratier] CHAR(35) NOT NULL, [Ra_ratierd] CHAR(30) NOT NULL)";
                    command.ExecuteNonQuery();
                    connection.Dispose();
                }

                catch (OleDbException ex)
                {
                    MessageBox.Show("Fehler beim Erzeugen der rates_S.dbf Datei: " + ex, "Hotel-Spider");
                }
                finally
                {
                    connection.Close();
                }
            }                          
        }


        // ODBC is also working  - but DBase3
 /*       public static void CreateDBFTableODBC(string path)
        {
            using (var dBaseConnection = new OdbcConnection(@"Driver={Microsoft dBASE Driver (*.dbf)};DriverID=277;Dbq=" + path + ";"))
            {
                try
                {
                    dBaseConnection.Open();

                    string str0 = "Create Table rates_S (Ra_ratcode char(8), Ra_priced char(30), Ra_curency char(4), Ra_ratier char(8), Ra_ratierd char(30))";
                    var cmd = new OdbcCommand(str0, dBaseConnection);
                    cmd.ExecuteNonQuery();
                }
                catch (OdbcException ex)
                {
                    MessageBox.Show("ODBC: " + ex, "Hotel-Spider");
                }
                finally
                {
                    dBaseConnection.Close();
                }
                
            }
        }
  */

    }
}
