﻿using System;
using System.Windows.Forms;
using System.IO;
using System.Data;
using Ifc;
using System.Xml;

namespace SpiderRates
{
    static class Program
    {
        /// <summary>
        /// The main entry point for the application.
        /// </summary>
        [STAThread]
        static void Main()
        {
            if (!File.Exists(Application.StartupPath + "\\IfcSettings.ini"))
            {
                MessageBox.Show("Die Datei 'IfcSettings.ini' wurde nicht gefunden. Ratecodes konnten nicht geladen werden. Bitte wenden sie sich an den Support.", "Hotel-Spider - Fehlermeldung!");
                HotelInterfaceTools.CloseApplication();
            }
            else
            {
                HotSpiderSettings.setConfiguration();
                Start();
            }

            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            Application.Run(new Form1());
        }

        public static void Start()
        {
            try
            {
                string pathDL = HotelInterfaceTools.getPath(HotSpiderSettings.localPath, "LOG");
                // Sende RQ an Hotel-Spider und empfange String
                string obtRatePlan = HotelInterfaceTools.SendRQandGetRS(SpiderRQ.RatePlans(), "https://extranet.hotel-spider.ch/ota/OTA_HotelRatePlanRQ/2014A");


                // fuer Testzwecke
                XmlDocument xmlDoc = new XmlDocument();
                xmlDoc.LoadXml(obtRatePlan);
                xmlDoc.Save(IfcSettings.ifcPath + "RatePlanAsRS.xml");
                

                // Falls der Empfang erfolgreich war, dann wird die Datei "RateCodeOK.dat" erzeugt
                if (HotelInterfaceTools.RSSuccess(obtRatePlan).Equals("Success"))
                {
                    // Make the retrieved XML-like string valid for parsing.
                    obtRatePlan = HotelInterfaceTools.MakeXmlValid(obtRatePlan);

                    // Bereite empfangenen XML-String für DataTable vor und speichere als XML
                    File.WriteAllText(HotSpiderSettings.ifcPath + "RateCodeInfo.xml", HotelInterfaceTools.ParseRatePlanXML(obtRatePlan));
                    
                    // Löscht Doppelte Elemente in den Ratecodes
                    HotelInterfaceTools.DeleteDoubleElements(HotSpiderSettings.ifcPath + "RateCodeInfo.xml");

                    // Lade die vorbereitete XML-Datei in DataSet
                    DataSet ds = new DataSet();
                    ds.ReadXml(HotSpiderSettings.ifcPath + "RateCodeInfo.xml");

                    // Wirte Dataset into DBF-file and CSV
                    if (HotSpiderSettings.oleDb.Equals("T") && HotSpiderSettings.rates.Equals("T"))
                    {
                        // lösche alte Rates_S.dbf
                        if (File.Exists(pathDL + "rates_S.dbf")) File.Delete(pathDL + "rates_S.dbf");
                                           
                        // rates_S.dbf wird erstellt
                        HotelInterfaceTools.CreateDBFTableOleDb(pathDL);                    
                        // Daten werden in rates_S.dbf geschrieben
                        HotelInterfaceTools.DataSetIntoDBF(pathDL, "RATES_S", ds);

                        // Zusätzlich wird CSV-Datei erstellt
                        HotelInterfaceTools.DataTableToCsv(ds.Tables[0], pathDL + "rates_S.csv");
                        string csvFile = File.ReadAllText(pathDL + "rates_S.csv");
                        File.WriteAllText(pathDL + "rates_S.csv", HotelInterfaceTools.ReplaceQuotes(csvFile));
                    }
                    else
                    {
                        // Es wird nur CSV-Datei erstellt
                        HotelInterfaceTools.DataTableToCsv(ds.Tables[0], pathDL + "rates_S.csv");
                        string csvFile = File.ReadAllText(pathDL + "rates_S.csv");
                        File.WriteAllText(pathDL + "rates_S.csv", HotelInterfaceTools.ReplaceQuotes(csvFile));
                    }
                    File.WriteAllText(pathDL + "RateCodeOK.dat", "Die Hotel-Spider Schnittstelle hat erfolgreich die Preiscodes aktuallisiert.");
                }
                else
                    File.Delete(pathDL + "RateCodeOK.dat");
                // Programm erfolgreich beenden
            } catch (Exception ex)
            {
                MessageBox.Show(ex.Message);
            }
            MessageBox.Show("Ratecodes wurden geladen.", "Hotel-Spider");

            HotelInterfaceTools.CloseApplication();                             
        }
    }
}
