﻿using System;
using System.ComponentModel;
using System.Diagnostics;
using System.Drawing;
using System.IO;
using System.Runtime.InteropServices;
using System.Windows.Forms;
using Ifc;
using System.ServiceProcess;
using IWshRuntimeLibrary;

namespace IfcWin
{
    public partial class IfcForm : Form
    { 
        // Use Console Windows for life protocol
        [DllImport("kernel32.dll")]
        private static extern bool AllocConsole();

        // Use Windows properties
        [DllImport("user32.dll")]
        public static extern bool ShowWindow(IntPtr hWnd, int nCmdShow);

        // Disable close button  
        private const int MF_BYCOMMAND = 0x00000000;
        public const int SC_CLOSE = 0xF060;

        [DllImport("user32.dll")]
        public static extern int DeleteMenu(IntPtr hMenu, int nPosition, int wFlags);

        [DllImport("user32.dll")]
        private static extern IntPtr GetSystemMenu(IntPtr hWnd, bool bRevert);

        [DllImport("kernel32.dll", ExactSpelling = true)]
        private static extern IntPtr GetConsoleWindow();

        private void InitializeConsole()
        {
            AllocConsole();
            Console.SetWindowSize(60, 30);
            Console.Title = Program.ifcName + " - Life Protokoll";
            
            // Disable close button
            DeleteMenu(GetSystemMenu(GetConsoleWindow(), false), SC_CLOSE, MF_BYCOMMAND);
            
            // Do not show Console on start
            IntPtr hideConsole = GetConsoleWindow();
            if (IntPtr.Zero != hideConsole)
            {
                ShowWindow(hideConsole, 0);
            }
        }

        private NotifyIcon notifyIcon;
        private ContextMenu contextMenu;
        private MenuItem menuItemMinimize;
        private MenuItem menuItemOpenApp;
        private MenuItem menuItemExecuteAction;
        private MenuItem menuItemLifeProtocol;
        private MenuItem menuItemOpenProtocol;
        private MenuItem menuItemEditProtocol;
        private MenuItem menuItemEditSettings;
        private MenuItem menuItemInfo;
        private MenuItem menuItemExit;
        private IContainer components = null;

        public IfcForm()
        {
            InitializeConsole();

            InitializeComponent();

            this.WindowState = FormWindowState.Normal;
            this.ShowInTaskbar = true;

            this.components = new Container();
            this.contextMenu = new ContextMenu();
            this.menuItemMinimize = new MenuItem();
            this.menuItemOpenApp = new MenuItem();
            this.menuItemExecuteAction = new MenuItem();
            this.menuItemLifeProtocol = new MenuItem();
            this.menuItemOpenProtocol = new MenuItem();
            this.menuItemEditProtocol = new MenuItem();
            this.menuItemEditSettings = new MenuItem();
            this.menuItemInfo = new MenuItem();
            this.menuItemExit = new MenuItem();

            // Initialize contextMenu 
            this.contextMenu.MenuItems.AddRange(
                        new MenuItem[] { this.menuItemMinimize });
            this.contextMenu.MenuItems.AddRange(
                        new MenuItem[] { this.menuItemOpenApp });
            this.contextMenu.MenuItems.AddRange(
                        new MenuItem[] { this.menuItemExecuteAction });
            this.contextMenu.MenuItems.AddRange(
                        new MenuItem[] { this.menuItemLifeProtocol });
            this.contextMenu.MenuItems.AddRange(
                        new MenuItem[] { this.menuItemOpenProtocol });
            this.contextMenu.MenuItems.AddRange(
                        new MenuItem[] { this.menuItemEditProtocol });
            this.contextMenu.MenuItems.AddRange(
                        new MenuItem[] { this.menuItemEditSettings });
            this.contextMenu.MenuItems.AddRange(
                        new MenuItem[] { this.menuItemInfo });           
            this.contextMenu.MenuItems.AddRange(
                        new MenuItem[] { this.menuItemExit });

            // Initialize menuItemMinimize
            this.menuItemMinimize.Index = 0;
            this.menuItemMinimize.Text = "Minimieren";
            this.menuItemMinimize.Click += new EventHandler(this.menuItemMinimize_Click);

            // Initialize menuItemMinimize
            this.menuItemOpenApp.Index = 1;
            this.menuItemOpenApp.Text = Program.ifcName + " öffnen";
            this.menuItemOpenApp.Click += new EventHandler(this.menuItemOpenApp_Click);

            // Initialize menuItemDoAction
            this.menuItemExecuteAction.Index = 2;
            this.menuItemExecuteAction.Text = "Daten Abrufen";
            this.menuItemExecuteAction.Click += new EventHandler(this.menuItemExecuteAction_Click);

            // Initialize menuItemLifeProtocol
            this.menuItemLifeProtocol.Index = 3;
            this.menuItemLifeProtocol.Text = "Life Protokoll";
            this.menuItemLifeProtocol.Click += new EventHandler(this.menuItemLifeProtocol_Click);

            // Initialize menuItemOpenProtocol
            this.menuItemOpenProtocol.Index = 4;
            this.menuItemOpenProtocol.Text = "Protokoll öffnen";
            this.menuItemOpenProtocol.Click += new EventHandler(this.menuItemOpenProtocol_Click);

            // Initialize menuItemEditProtocol
            this.menuItemEditProtocol.Index = 5;
            this.menuItemEditProtocol.Text = "Protokoll bearbeiten";
            this.menuItemEditProtocol.Click += new EventHandler(this.menuItemEditProtocol_Click);

            // Initialize menuItemEditSettings
            this.menuItemEditSettings.Index = 6;
            this.menuItemEditSettings.Text = "Einstellungen bearbeiten";
            this.menuItemEditSettings.Click += new EventHandler(this.menuItemEditSettings_Click);

            // Initialize menuItemInfo
            this.menuItemInfo.Index = 7;
            this.menuItemInfo.Text = "Info";
            this.menuItemInfo.Click += new EventHandler(this.menuItemInfo_Click);

            // Initialize menuItemExit 
            this.menuItemExit.Index = 8;
            this.menuItemExit.Text = "Beenden";
            this.menuItemExit.Click += new EventHandler(this.menuItemExit_Click);

            // Set up how the form should be displayed. 
            this.ClientSize = new Size(540, 270);
            this.Text = Program.ifcName;

            // Create the NotifyIcon. 
            this.notifyIcon = new NotifyIcon(this.components);

            // The Icon property sets the icon that will appear 
            // in the systray for this application.
            notifyIcon.Icon = new Icon(IfcSettings.iconFile);
            
            // The ContextMenu property sets the menu that will 
            // appear when the systray icon is right clicked.
            notifyIcon.ContextMenu = this.contextMenu;

            // The Text property sets the text that will be displayed, 
            // in a tooltip, when the mouse hovers over the systray icon.
            notifyIcon.Text = Program.ifcName;
            notifyIcon.Visible = true;

            // Handle the DoubleClick event to activate the form.
            notifyIcon.DoubleClick += new EventHandler(this.notifyIcon_DoubleClick);           
        }

        protected override void Dispose(bool disposing)
        {
            // Clean up any components being used. 
            if (disposing)
                if (components != null)
                    components.Dispose();

            base.Dispose(disposing);
        }

        protected override void OnFormClosing(FormClosingEventArgs e)
        {
            base.OnFormClosing(e);
             
            DialogResult exitMsg = MessageBox.Show("Möchten sie " + Program.ifcName + " wirklich beenden?",
                "Warnung!",
                MessageBoxButtons.YesNo);
            if (exitMsg == DialogResult.Yes)
            {
                IfcLog.WriteLog(IfcSettings.logFile, Program.ifcName + " wurde beendet.");
                Dispose(true);
                Program.CloseApplication();
            }
            else
            {
                e.Cancel = true;
            }
        }
       
        protected override void WndProc(ref Message m)
        {
            if (m.Msg == /*WM_SIZE*/ 0x0005)
            {
                if (this.WindowState == FormWindowState.Minimized)
                {
                    this.ShowInTaskbar = false;
                    ShowWindow(GetConsoleWindow(), 0);
                }
            }
            base.WndProc(ref m);
        }

        // Show the form when the user double clicks on the notify icon. 
        private void notifyIcon_DoubleClick(object Sender, EventArgs e)
        {
            // Set the WindowState to normal if the form is minimized, otherwise minimize it. 
            if (this.WindowState == FormWindowState.Minimized)
            {
                this.WindowState = FormWindowState.Normal;
                this.ShowInTaskbar = true;
            }
            else
            {
                this.WindowState = FormWindowState.Minimized;
            }

            // Activate the form. 
            this.Activate();
        }

        private void menuItemMinimize_Click(object Sender, EventArgs e)
        {
            this.WindowState = FormWindowState.Minimized;
            ShowWindow(GetConsoleWindow(), 0);
        }

        private void menuItemOpenApp_Click(object Sender, EventArgs e)
        {
            this.WindowState = FormWindowState.Normal;
        }

        private void menuItemExecuteAction_Click(object Sender, EventArgs e)
        {
            Program.IntervalAction();
            FillLogTextBox();
        }

        private void menuItemLifeProtocol_Click(object Sender, EventArgs e)
        {
            ShowWindow(GetConsoleWindow(), 1);
        }

        private void menuItemOpenProtocol_Click(object Sender, EventArgs e)
        {
            this.WindowState = FormWindowState.Normal;
            FillLogTextBox();
        }

        private void menuItemEditProtocol_Click(object Sender, EventArgs e)
        {
            if (PasswordDialog("Passwort:", Program.ifcName + " - Protokoll bearbeiten").Equals(IfcTools.Password()))
            {
                Process.Start(IfcSettings.logFile);
            }
        }

        private void menuItemEditSettings_Click(object Sender, EventArgs e)
        {
            if (PasswordDialog("Passwort:", Program.ifcName + " - Einstellungen bearbeiten").Equals(IfcTools.Password()))
            {
                Process.Start(IfcSettings.settingsFile);
            }           
        }

        private void menuItemInfo_Click(object Sender, EventArgs e)
        {
            ShowInfoBox();
        }

        private void menuItemExit_Click(object Sender, EventArgs e)
        {
            this.Close();
        }      

        private void actionToolStripMenuItem_Click(object sender, EventArgs e)
        {
            Program.IntervalAction();
            FillLogTextBox();
        }

        private void lifeProtokollToolStripMenuItem_Click(object sender, EventArgs e)
        {
            ShowWindow(GetConsoleWindow(), 1);
        }

        private void protokollÖffnenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            FillLogTextBox();
        }

        private void protokollBearbeitenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (PasswordDialog("Passwort:", Program.ifcName + " - Protokoll bearbeiten").Equals(IfcTools.Password()))
            {
                Process.Start(IfcSettings.logFile);
            }
        }

        private void einstellungenBearbeitenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (PasswordDialog("Passwort:", Program.ifcName + " - Einstellungen bearbeiten").Equals(IfcTools.Password()))
            {
                Process.Start(IfcSettings.settingsFile);
            }
        }

        private void beendenToolStripMenuItem_Click(object sender, EventArgs e)
        {
            this.Close();
        }

        private void infoToolStripMenuItem_Click(object sender, EventArgs e)
        {
              ShowInfoBox();
        }

        private static string PasswordDialog(string text, string caption)
        {
            Form prompt = new Form();
            prompt.Width = 320;
            prompt.Height = 160;
            prompt.FormBorderStyle = FormBorderStyle.FixedDialog;
            prompt.Text = caption;
            prompt.MaximizeBox = false;
            prompt.MinimizeBox = false;
            prompt.StartPosition = FormStartPosition.CenterScreen;
            Label textLabel = new Label() { Left = 50, Top = 20, Text = text };
            TextBox textBox = new TextBox() { Left = 50, Top = 50, Width = 200 };
            textBox.Text = "";
            textBox.PasswordChar = '*';
            textBox.MaxLength = 14;
            Button confirmation = new Button() { Text = "Ok", Left = 150, Width = 100, Top = 80, DialogResult = DialogResult.OK };
            confirmation.Click += (sender, e) => { prompt.Close(); };
            prompt.Controls.Add(textBox);
            prompt.Controls.Add(confirmation);
            prompt.Controls.Add(textLabel);
            prompt.AcceptButton = confirmation;

            return prompt.ShowDialog() == DialogResult.OK ? textBox.Text : "";
        }

        private void ShowInfoBox()
        {
            AboutBox infoBox = new AboutBox();
            infoBox.Show();
        }

        private void FillLogTextBox()
        {
            logTextBox.Text = IfcLog.ReverseLines(IfcSettings.logFile);
        }

        private void timer1_Tick(object sender, EventArgs e)
        {
            // Bei Monatswechsel wird die Log-Datei archiviert
            IfcLog.CheckLogFile(IfcSettings.logFileName);
            if (!windowsServiceIsRunning()) Program.IntervalAction();
        }

        private void autostartMenuItem_Click(object sender, EventArgs e)
        {
            if (PasswordDialog("Passwort:", Program.ifcName + " - Autostart bearbeiten").Equals(IfcTools.Password()))
            {
                try
                {
                    string startFolderPath = Environment.GetFolderPath(Environment.SpecialFolder.Startup);

                    if (autostartMenuItem.Checked == true)
                    {
                        if (System.IO.File.Exists(startFolderPath + "\\SpiderRes.lnk"))
                        {
                            System.IO.File.Delete(startFolderPath + "\\SpiderRes.lnk");
                        }

                        if (!System.IO.File.Exists(startFolderPath + "\\SpiderRes.lnk"))
                            autostartMenuItem.Checked = false;
                    }
                    else
                    {
                        if (System.IO.File.Exists(System.IO.Directory.GetCurrentDirectory() + "\\SpiderRes.exe") &&
                             System.IO.Directory.Exists(startFolderPath))
                        {
                            WshShell shell = new WshShell();
                            string shortcutAddress = startFolderPath + "\\SpiderRes.lnk";
                            IWshShortcut shortcut = (IWshShortcut)shell.CreateShortcut(shortcutAddress);
                            shortcut.TargetPath = System.IO.Directory.GetCurrentDirectory() + "\\SpiderRes.exe";
                            shortcut.WorkingDirectory = System.IO.Directory.GetCurrentDirectory();
                            shortcut.Save();
                        }

                        if (System.IO.File.Exists(startFolderPath + "\\SpiderRes.lnk"))
                            autostartMenuItem.Checked = true;
                    }
                }
                catch (Exception ex) { MessageBox.Show(ex.Message, Program.ifcName); }
            }           
        }

        private void windowsServiceMenuItem_Click(object sender, EventArgs e)
        {
            using (ServiceController sc = new ServiceController("IfcResService"))
            {
                try
                {
                    if (sc.Status == ServiceControllerStatus.Running)
                    {
                        windowsServiceMenuItem.Text = "Service wurde gestartet";
                        windowsServiceMenuItem.ForeColor = Color.Green;
                    }
                    else
                    {
                        windowsServiceMenuItem.Text = "Service wurde nicht gestartet";
                        windowsServiceMenuItem.ForeColor = Color.Black;
                    }
                }
                catch (InvalidOperationException)
                {
                    windowsServiceMenuItem.Text = "Service ist nicht installiert";
                    windowsServiceMenuItem.ForeColor = Color.Black;
                }
            }
        }

        private void IfcForm_Load(object sender, EventArgs e)
        {
            // Check at program start if shortcut of App exists in startup folder. If yes thes specific menu is checked
            if (System.IO.File.Exists(Environment.GetFolderPath(Environment.SpecialFolder.Startup) + "\\SpiderRes.lnk"))
                autostartMenuItem.Checked = true;

            // Windows Service checken
            using (ServiceController sc = new ServiceController("IfcResService"))
            {
                try
                {
                    if (sc.Status == ServiceControllerStatus.Running)
                    {
                        windowsServiceMenuItem.Text = "Service wurde gestartet";
                        windowsServiceMenuItem.ForeColor = Color.Green;
                    }
                    else
                    {
                        windowsServiceMenuItem.Text = "Service wurde nicht gestartet";
                        windowsServiceMenuItem.ForeColor = Color.Black;
                    }
                }
                catch (InvalidOperationException)
                {
                    windowsServiceMenuItem.Text = "Service ist nicht installiert";
                    windowsServiceMenuItem.ForeColor = Color.Black;
                }
            }
        }

        private bool windowsServiceIsRunning()
        {
            using (ServiceController sc = new ServiceController("IfcResService"))
            {
                try
                {
                    if (sc.Status == ServiceControllerStatus.Running) return true;
                    else return false;
                }
                catch (InvalidOperationException)
                {
                    return false;
                }
            }
        }

        private void confirmMenuItem_Click(object sender, EventArgs e)
        {
            if (PasswordDialog("Passwort:", Program.ifcName + " - Reservierungen bestätigen").Equals(IfcTools.Password()))
            {
                if (confirmMenuItem.Checked == true)
                {
                    Program.confirmation = false;
                    confirmMenuItem.Checked = false;
                }
                else
                {
                    Program.confirmation = true;
                    confirmMenuItem.Checked = true;
                }

            }
        }

        private void singleConfirmToolStripMenuItem_Click(object sender, EventArgs e)
        {
            if (PasswordDialog("Passwort:", Program.ifcName + " - Reservierungen einzeln bestätigen").Equals(IfcTools.Password()))
            {
                if (singleConfirmToolStripMenuItem.Checked == true)
                {
                    Program.singleConfirmation = false;
                    singleConfirmToolStripMenuItem.Checked = false;
                }
                else
                {
                    Program.singleConfirmation = true;
                    singleConfirmToolStripMenuItem.Checked = true;
                }

            }
        }
    }
}