﻿using System;
using System.Windows.Forms;
using System.IO;
using System.Xml.Linq;
using Ifc;
using System.Text;
using System.Xml;
using System.Net;

namespace IfcRooms
{
    static class Program
    {
        // Interface Name
        public static string name = "Spider";
        public static string ifcName = name + " Hotel-Interface";

        /// <summary>
        /// Der Haupteinstiegspunkt für die Anwendung.
        /// </summary>
        [STAThread]
        static void Main()
        {
            if (!File.Exists(IfcSettings.settingsFile))
            {
                MessageBox.Show("Die Datei mit den Programmeinstellungen wurde nicht gefunden. Preiscodes konnten nicht geladen werden. Bitte wenden sie sich an den Support.", ifcName);
                IfcLog.WriteLog(IfcSettings.logFile, "Die Datei: " + IfcSettings.settingsFile + " ist nicht vorhanden!");
                CloseApplication();
            }
            else
            {
                if (IfcNet.DnsTestOK(IfcSettings.dnsTestName))
                {
                    IfcSettings.setConfiguration(IfcSettings.settingsFile);
                    Start();
                }
                else
                {
                    MessageBox.Show("Sie haben zur Zeit keine Internetverbindung. Überprüfen sie bitte ihren Internetanschluss.", ifcName);
                    CloseApplication();
                }
            }

            Application.EnableVisualStyles();
            Application.SetCompatibleTextRenderingDefault(false);
            Application.Run(new Form1());
        }

        // Für vorzeitiges Beenden des Programms
        private static void CloseApplication()
        {
            if (Application.MessageLoop)
            {
                // WinForms app
                Application.Exit();
            }
            else
            {
                // Console app
                Environment.Exit(1);
            }
        }

        private static void Start()
        {
            string obtRooms = "";
            try
            {
                //// Test
                //string url = "https://validation.hotel-spider.com/tls";

                // Sende RQ an Hotel-Spider und empfange String
                string url = "https://extranet.hotel-spider.ch/ota/OTA_HotelDescriptiveInfoRQ/2014A";

                obtRooms = IfcNet.SendRQandGetRS(IfcOTAMessage.OTA_HotelDescriptiveInfoRQ(), url, IfcSettings.logFile);
                
                File.WriteAllText(IfcSettings.ifcPath + "IfcRoomsInfo.info", IfcOTA.ParseRoomsXML(obtRooms, IfcSettings.logFile));
            }
            catch (Exception ex)
            {
                IfcLog.WriteLog(IfcSettings.logFile, ex);
                MessageBox.Show("Ein Problem beim Empfang der Zimmerinformationen ist aufgetreten.", ifcName);
                CloseApplication();
            }
            MessageBox.Show("Aktuelle Zimmerinformationen wurden erfolgreich geladen.", ifcName);

            // Programm erfolgreich beenden            
            CloseApplication();
        }
    }
}
